package TestPaymentMethodBillsafe;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use Test::More;
use Test::MockObject;
use Test::Exception;
use DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            testIsAvailableInCountry();
            testIsAvailableForCurrency();
            testGetCountryFromLineItemContainer();
            testCanBillSafeAddToBasket();
            done_testing();
        }
    );
}

sub testIsAvailableInCountry {
    my $subTestName = (caller(0))[3];
    my $PaymentMethod = DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe->new();

    is ($PaymentMethod->isAvailableInCountry(undef),1,"$subTestName - Checkes if this works with undef Parameter ( extra behavoir for the classic checkout)- True");

    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = undef;
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'InjectedCode3'}),0,"$subTestName - Checkes if this works with undef AvailableInCountry array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = [];
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'InjectedCode3'}),0,"$subTestName - Checkes if this works with empty AvailableInCountry array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['InjectedCode3'];
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'InjectedCode3'}),1,"$subTestName - Checkes if this works with one Element in AvailableInCountry array - True");
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'InjectedCode2'}),0,"$subTestName - Checkes if this works with one Element which is not in AvailableInCountry array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['FirstInjectedCode3','SecondInjectedCode3'];
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'SecondInjectedCode3'}),1,"$subTestName - Checkes if this works with multible Elements in AvailableInCountry array - True");
    is ($PaymentMethod->isAvailableInCountry({Code3 => 'FirstInjectedCode3'}),1,"$subTestName - Checkes if this works with multible Elements in AvailableInCountry array - True");
}

sub testIsAvailableForCurrency {
    my $subTestName = (caller(0))[3];
    my $PaymentMethod = DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe->new();

    is ($PaymentMethod->isAvailableForCurrency(undef),0,"$subTestName - Checkes if this works with undef Parameter - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = undef;
    is ($PaymentMethod->isAvailableForCurrency('EUR'),0,"$subTestName - Checkes if this works with undef AvailableCurrencies array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = [];
    is ($PaymentMethod->isAvailableForCurrency('ACurrency'),0,"$subTestName - Checkes if this works with empty AvailableCurrencies array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['ACurrency'];
    is ($PaymentMethod->isAvailableForCurrency('ACurrency'),1,"$subTestName - Checkes if this works with one Element in AvailableCurrencies array - True");
    is ($PaymentMethod->isAvailableForCurrency('XXX'),0,"$subTestName - Checkes if this works with one Element which is not in AvailableCurrencies array - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['ACurrency','OtherCurrency'];
    is ($PaymentMethod->isAvailableForCurrency('OtherCurrency'),1,"$subTestName - Checkes if this works with multible Elements in AvailableCurrencies array - True");
    is ($PaymentMethod->isAvailableForCurrency('ACurrency'),1,"$subTestName - Checkes if this works with multible Elements in AvailableCurrencies array - True");
}


sub testGetCountryFromLineItemContainer {
    my $subTestName = (caller(0))[3];
    my $PaymentMethod = DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe->new();
    throws_ok (
        sub { $PaymentMethod->getCountryFromLineItemContainer() },
        'DE_EPAGES::Core::API::Error',
        "$subTestName - Checkes the behavior of undefined lineItemContiner - epages core Error API Error"
    );
    throws_ok (
        sub { $PaymentMethod->getCountryFromLineItemContainer(_GetBaseObjectMock()) },
        'DE_EPAGES::Core::API::Error',
        "$subTestName - Checkes the behavior of undefined BillingCountryID Attribute - epages core Error API Error"
    );
    is_deeply(
        $PaymentMethod->getCountryFromLineItemContainer(_GetBaseObjectMock({'BillingCountryID'=>undef})),
        {},
        "$subTestName - Checkes the behavior of undefined BillingCountryID value - empty hash ref"
    );
    my $GermanCountryID = 276;## no critic ( ProhibitMagicNumbers )
    my $Country = $PaymentMethod->getCountryFromLineItemContainer(_GetBaseObjectMock({'BillingCountryID'=>$GermanCountryID}));
    is(
        $Country->{'Code3'},
        'DEU',
        "$subTestName - Checkes if getCountryFromLineItemContainer returns the expected Country hash - 'DEU'"
    );
}

sub testCanBillSafeAddToBasket{
    my $subTestName = (caller(0))[3];
    my $PaymentMethod = DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe->new();
    my $GermanCountryID = 276;## no critic ( ProhibitMagicNumbers )
    my $hVarsMock = {
        'LineItemContainer'=>_GetBaseObjectMock({'BillingCountryID'=>$GermanCountryID}),
        'CurrencyID' => 'EUR'
    };

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['DKK'];
    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['DEU'];
    is ($PaymentMethod->canBillSafeAddToBasket($hVarsMock),0,"$subTestName - Check canBillSafeAddToBasket for condition [wrong currency,correct country]- False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['EUR'];
    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['FIN'];
    is ($PaymentMethod->canBillSafeAddToBasket($hVarsMock),0,"$subTestName - Check canBillSafeAddToBasket for condition [correct currency, wrong country] - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['DKK'];
    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['FIN'];
    is ($PaymentMethod->canBillSafeAddToBasket($hVarsMock),0,"$subTestName - Check canBillSafeAddToBasket for condition [wrong currency, wrong country] - False");

    $PaymentMethod->{'Attributes'}{'AvailableCurrencies'} = ['EUR'];
    $PaymentMethod->{'Attributes'}{'AvailableCountries'} = ['DEU'];
    is ($PaymentMethod->canBillSafeAddToBasket($hVarsMock),1,"$subTestName - Check canBillSafeAddToBasket for condition [correct currency, correct country] - True");
}
sub _GetBaseObjectMock {
    my ($hAtributes) = @_;
    return  DE_EPAGES::Core::API::Object::BaseObject->new(%{$hAtributes});
}
RunScript( Sub => \&Main );