package TransactionTest;
use base DE_EPAGES::Database::API::Test;

use Test::More tests => 11;

use strict;
use DE_EPAGES::Database::API::Connection qw ( Transaction );
use DE_EPAGES::Core::API::Error qw (Error ExistsError);

sub test {
    my $self = shift;

    my $dbi = $self->dbi;
    $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
    Transaction(sub {
        $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
        $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
    });
    my $aaResults = $dbi->execute('select id from dbitest');
    ok (3 == scalar @$aaResults, 'transaction without error');
    $dbi->do('delete from dbitest');

    # error
    $aaResults = $dbi->execute('select id from dbitest');
    ok (0 == scalar @$aaResults, 'transaction with error - reset ');
    eval {
        $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
        Transaction(sub {
            $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
            die('Transaction crash');
        });
    };
    ok( ExistsError(), 'transaction aborted after INSERT' );
    $aaResults = $dbi->execute('select id from dbitest');
    ok (1 == scalar @$aaResults, 'transaction with delete error ');
    $dbi->do('delete from dbitest');

    # error
    $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
    $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);

    eval {
        Transaction(sub {
            $dbi->do('delete from dbitest');
            $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
            die('Transaction crash');
        });
    };
    ok( ExistsError(), 'transaction aborted after DELETE+INSERT' );
    $aaResults = $dbi->execute('select id from dbitest');
    ok (2 == scalar @$aaResults, 'transaction with die');
    $dbi->do('delete from dbitest');

    # error + nested transaction
    eval {
        $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
        Transaction(sub {
            $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
            Transaction(sub {
                $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
            });
            Error('SoNicht');
        });
    };
    ok( ExistsError(), 'transaction aborted after sub transaction' );
    $aaResults = $dbi->execute('select id from dbitest');
    ok (1 == scalar @$aaResults, 'transaction with error');
}

sub init {
    my $self = shift;

    my $dbi = $self->dbi;
    $dbi->do( 'CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL)' );

    ok( $dbi->existsTableColumn( 'dbitest', 'name' ), 'init' );
}

sub cleanup {
    my $self = shift;

    my $dbi = $self->dbi;
    $dbi->do( 'DROP TABLE dbitest' ) if $dbi->existsTable( 'dbitest' );
    ok( 1, 'cleanup' );
}

__PACKAGE__->runScript;

1;
