use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use Test::More tests => 8;

use DE_EPAGES::EPagesJ::API::EPagesJ qw ( GetInfoClient IsEPagesJVersion );

#========================================================================================
#   MAIN
#========================================================================================
sub Test {
    isa_ok(GetInfoClient(), 'DE_EPAGES::EPagesJ::API::Object::InfoClient');
    # ePagesJ version matches ePages6
    my $EPJVersionRunning = GetInfoClient()->version;
    ok(IsEPagesJVersion($EPJVersionRunning), 'IsEPagesJVersion -- expected == running' );

    my $Expected = '1.1.1';
    ok(!IsEPagesJVersion($Expected), 'IsEPagesJVersion -- expected < running' );
    ok(IsEPagesJVersion($Expected, {HigherVersionIsTrue => 1}), 'IsEPagesJVersion -- expected < running, flag HigherVersionIsTrue=1' );
    ok(!IsEPagesJVersion($Expected, {HigherVersionIsTrue => 0}), 'IsEPagesJVersion -- expected < running, flag HigherVersionIsTrue=0' );

    # older version of ePagesJ is running
    $Expected = '99.99.99';
    ok(!IsEPagesJVersion($Expected), 'IsEPagesJVersion -- expected > running' );

    # newer version of ePagesJ is running
    $Expected = '6.15.0';
    ok(IsEPagesJVersion($Expected, {HigherVersionIsTrue => 1}), 'IsEPagesJVersion -- expected > running; flag HigherVersionIsTrue=1' );
    ok(!IsEPagesJVersion($Expected, {HigherVersionIsTrue => 0}), 'IsEPagesJVersion -- expected > running; flag HigherVersionIsTrue=0' );
}

RunScript(
    Sub => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => \&Test
        );
    }
);
