/*jslint browser:true*/
/*global define*/
define(["testsuite/qunit", "jquery", "ep/fn/contextorientation", "$ready!"], function (QUnit, $, ep) {

    "use strict";

    var elem, context;

    QUnit.module("ep/fn/contextorientation", {
        setup: function () {
            elem = $("#test-elem");
            context = $("#test-context");

            context.css({
                position: "absolute",
                width: 50,
                height: 50
            });
        }
    });

    QUnit.test("Context and element are in viewport", 5, function () {
        context.css({
            top: 200,
            left: 500
        });

        elem.css({
            width: 150,
            height: 150
        });

        elem.contextOrientation(context, "top");

        QUnit.deepEqual(
            elem.position(),
            { left: 500, top: 50 },
            "Element is aligned to the top of the context"
        );

        elem.contextOrientation(context, "left bottom");

        QUnit.deepEqual(
            elem.position(),
            { left: 350, top: 250 },
            "Element is aligned to the lower left of the context"
        );

        elem.contextOrientation(context, "right top");

        QUnit.deepEqual(
            elem.position(),
            { left: 550, top: 50 },
            "Element is aligned to the upper right of the context"
        );

        elem.contextOrientation(context, "center middle");

        QUnit.deepEqual(
            elem.position(),
            { left: 450, top: 150 },
            "Element is centered on the context"
        );

        elem.contextOrientation(context, "center");

        QUnit.deepEqual(
            elem.position(),
            { left: 450, top: 200 },
            "Element is centered under the context"
        );
    });

    QUnit.test("Element would be outside of the viewport", 2, function () {
        context.css({
            top: 10,
            left: 10
        });

        elem.css({
            width: 50,
            height: 50
        });

        elem.contextOrientation(context, "top left");

        QUnit.deepEqual(
            elem.position(),
            { left: 60, top: 60 },
            "Element is aligned to the lower right of the context"
        );

        context.css({
            top: $(window).height() - 10,
            left: $(window).width() - 10
        });

        elem.contextOrientation(context, "bottom right");

        QUnit.deepEqual(
            elem.position(),
            { left: context.position().left - 50, top: context.position().top - 50 },
            "Element is aligned to the upper left of the context"
        );
    });

    QUnit.test("Context is an instance of $.Event", 1, function () {
        var evt = new $.Event();

        elem.css({
            width: 50,
            height: 50
        });

        evt.pageX = 500;
        evt.pageY = 10;

        elem.contextOrientation(evt, "top right");

        QUnit.deepEqual(
            elem.position(),
            { left: 500, top: 10 },
            "Element is aligned to the lower right of the context"
        );
    });

});
