/*global define, window*/
/*jslint nomen:true*/

define([
    'testsuite/qunit',
    'jquery',

    'ep/ui/smartsearch',
    '$ready!'
], function (QUnit, $) {
    'use strict';

    var _ajax = $.ajax,
        searchResults = {
            Results: [{
                name: 'Mars',
                alias: 'Mars'
            }, {
                name: 'Snickers',
                alias: 'Snickers'
            }, {
                name: 'Bounty',
                alias: 'Bounty'
            }, {
                name: 'Twix',
                alias: 'Twix'
            }]
        },
        localSource = [
            'Affe', 'Ente', 'Giraffe', 'Wal'
        ],
        smartLocal,
        smartLocal2,
        smartRemote;

    $.ajax = function (o) {
        var result = $.Deferred();

        if (o.data) {
            if (o.data.ViewAction === "JSONGetSearchResults") {
                result.resolve(searchResults);
            }
        } else {
            return _ajax(o);
        }
        return result;
    };

    QUnit.module("ep/ui/smartsearch", {
        setup: function () {
            smartLocal = $('#ep-ui-smart-local').uiSmartsearch({
                source: localSource,
                multiple: true,
                group: 'Tiere'
            });

            smartLocal2 = $('#ep-ui-smart-local2').uiSmartsearch({
                source: localSource,
                multiple: true,
                group: 'Tiere'
            });

            smartRemote = $('#ep-ui-smart-remote').uiSmartsearch({
                source: 'JSONGetSearchResults',
                multiple: false
            });
        },
        teardown: function () {
            $('.ui-autocomplete').remove();
        }
    });

    QUnit.test('Check smartsearch methods', 4, function () {
        $.each(['_create', '_init', '_checkTags', 'destroy'], function () {
            QUnit.strictEqual(
                $.isFunction($.ui.uiSmartsearch.prototype[this]),
                true,
                this + ' is a method of ui smartsearch'
            );
        });
    });

    QUnit.test('Check destroy method', function () {

        smartLocal.uiSmartsearch('destroy');
        smartLocal2.uiSmartsearch('destroy');
        smartRemote.uiSmartsearch('destroy');
        QUnit.strictEqual(
            $('.ui-autocomplete').length,
            0,
            'Check if all smartsearch widgets have been destroyed'
        );
    });

    QUnit.asyncTest('Check local autocomplete', function () {
        QUnit.strictEqual(
            smartLocal.hasClass('ui-autocomplete-input'),
            true,
            'Autocomplete was created'
        );

        smartLocal.val('ff').trigger('keydown');

        window.setTimeout(function () {
            QUnit.strictEqual(
                $('.ui-autocomplete:first').children().length,
                2,
                'Autocomplete should return 2 results'
            );

            $('.ui-autocomplete:first a:first').trigger('click');
            QUnit.strictEqual(
                smartLocal.val(),
                'Affe, ',
                '"Affe, " should be inserted (multiple)'
            );

            smartLocal.val('Zwerghamster').trigger('keydown').trigger('change').trigger('blur');
            smartLocal2.val('Z').trigger('keydown');
            window.setTimeout(function () {
                QUnit.strictEqual(
                    $($('.ui-autocomplete')[1]).children().length,
                    1,
                    'Grouped autocomplete should return 1 result'
                );

                $($('.ui-autocomplete')[1]).find('a:first').trigger('click');
                QUnit.strictEqual(
                    smartLocal2.val(),
                    'Zwerghamster, ',
                    '"Zwerghamster, " should be inserted (multiple)'
                );

                QUnit.start();
            }, 500);
        }, 500);

    });

    QUnit.asyncTest('Check remote autocomplete', function () {
        smartRemote.val('i').trigger('keydown');

        window.setTimeout(function () {
            QUnit.strictEqual(
                $('.ui-autocomplete:last').children().length,
                4,
                'Autocomplete should return 4 results'
            );

            $('.ui-autocomplete:last a:last').trigger('click');
            QUnit.strictEqual(
                smartRemote.val(),
                'Twix',
                '"Twix" should be inserted'
            );
            QUnit.start();
        }, 500);

    });
});