use Test::More tests => 5;
use strict;

use DE_EPAGES::Core::API::AutoFollowUserAgent;
use DE_EPAGES::Core::API::File  qw (DeleteDirectory);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);

my $SHOPPATH = '/Shops/DemoShop';

sub checkResponse {
    my ($Browser, $URL) = @_;
    my $LoadPage = $Browser->get($URL);

    my $Result = [-1,''];

    if($LoadPage->is_success()){
        my $Content = $LoadPage->content();
        my $Match = $Content =~ /ViewAction=SF-ViewJQueryConfig.*?-([0-2])">/xgs ? $1 : undef;
        $Result = [$Match, $LoadPage->header('content_base')];
    }

    return $Result;
}

sub TestSetSessionDebug {
    my ($Shop) = @_;

    my $SystemDebugLvl = LoadRootObject()->get('JSDebugLevel');

    my $ShopURL = $Shop->get('WebServerScriptUrl');
    my $Browser = DE_EPAGES::Core::API::AutoFollowUserAgent->new;

    my ($DebugLvL,$SessionURL) = @{checkResponse($Browser,$ShopURL)};
    is($DebugLvL,$SystemDebugLvl, "System JSDebugLevel: $SystemDebugLvl" );

    ($DebugLvL,$SessionURL) = @{checkResponse($Browser,$SessionURL.'?ChangeAction=SetDebugSession&Debug=1')};
    is($DebugLvL,1, 'JSDebugLevel after change: 1' );

    is(${checkResponse($Browser,$SessionURL)}[0],1, 'JSDebugLevel after reload: 1' );

    is(${checkResponse($Browser,$SessionURL.'&ChangeAction=SetDebugSession')}[0],$SystemDebugLvl, "JSDebugLevel after reset: $SystemDebugLvl" );
}

sub Cleanup {
    DeleteDirectory( 'ctmpl', { recursive => 1 } );
}

sub Main {
    Cleanup();
    if( ExistsObjectByPath($SHOPPATH) ) {
        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, 'load demoshop ' . $Shop->id );
        TestSetSessionDebug($Shop);
    } else {
        SKIP: {
            skip 'no DemoShop installed', 5;
        };
    }
    Cleanup();
    return;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
